﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.Collections;
using Microsoft.DirectX;
using Microsoft.DirectX.AudioVideoPlayback;
using System.Windows.Forms;

namespace MP3Player
{
    public enum PlayState
    {
        Stopped = 0,
        Paused = 1,
        Running = 2,
        NoData = 3
    }

    class PlayBack : IDisposable
    {
        private const long DoubleToTicks = 10000000;
        private bool disposed = false;

        private Audio _audio;
        private Video _video;
        private Control _videoWnd;
        private String _fileName;
        private int _volume;

        public PlayBack(Control videoWnd)
        {
            _videoWnd = videoWnd;
        }

        public PlayBack(String fileName, Control videoWnd)
        {
            _videoWnd = videoWnd;
            FileName = fileName;
        }

        public string FileName
        {
            get
            {
                return _fileName;
            }
            set
            {
                _fileName = value;
                //the DirectX Audio/Video classes must be explicitly disposed to cancel playback
                if (_video != null)
                    _video.Dispose();
                _video = null;
                if (_audio != null)
                    _audio.Dispose();
                _audio = null;
                if (_fileName == "")
                {
                    return;
                }
                try
                {
                    _video = new Video(_fileName);
                    _video.Owner = _videoWnd;
                    _audio = _video.Audio;
                }
                catch (DirectXException)
                {
                    try
                    {
                        _video = null;
                        _audio = new Audio(_fileName);
                    }
                    catch (DirectXException)
                    {
                        _audio = null;
                        throw;
                    }
                }
            }
        }

        public PlayState State
        {
            get
            {
                if (_audio == null)
                    return PlayState.NoData;
                return (PlayState)_audio.State;
            }
        }

        public int Volume
        {
            get
            {
                return _volume;
            }
            set
            {
                _volume = value;
                if (_audio != null)
                    _audio.Volume = _volume;
            }
        }

        public void Play()
        {
            if (_audio == null)
                throw new AudioVideoException();
            _audio.Play();
        }

        public void Stop()
        {
            if (_audio != null)
                _audio.Stop();
        }

        public void Pause()
        {
            if (_audio == null)
                throw new AudioVideoException();
            _audio.Pause();
        }

        public long CurrentPosition
        {
            get
            {
                if (_audio == null)
                    return 0;
                long pos = (long)_audio.CurrentPosition * DoubleToTicks;
                //we don't want to report a position larger than the duration
                if (pos > Duration)
                    return Duration;
                return pos;
            }
            set
            {
                if (_audio == null)
                    throw new AudioVideoException();
                if ((double)value / DoubleToTicks > _audio.Duration)
                    throw new ArgumentException("Position must be >= 0 and <= Duration");
                _audio.CurrentPosition = (double)value / DoubleToTicks;
            }
        }

        public long Duration
        {
            get
            {
                if (_audio == null)
                    return 0;
                return (int)_audio.Duration * DoubleToTicks;
            }
        }

        public bool HasVideo
        {
            get
            {
                return (_video != null);
            }

        }

        private void Dispose(bool disposing)
        {
            if (!this.disposed)
            {
                if (disposing)
                {
                    if (_audio != null)
                        _audio.Dispose();
                    if (_video != null)
                        _video.Dispose();
                }
                disposed = true;
            }
        }

        ~PlayBack()
        {
            Dispose(false);
        }

        public void Dispose()
        {
            Dispose(true);
            GC.SuppressFinalize(this);
        }
    }

    public class AudioVideoException : InvalidOperationException 
    {
        public AudioVideoException()
            : base("There is no valid audio/video file loaded.")
        {
        }

        public AudioVideoException(string message)
            : base(message)
        {
        }
    }
}
