﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Windows.Forms.VisualStyles;

namespace MP3Player
{
    [DefaultBindingPropertyAttribute("Value")]
    public partial class Slider : UserControl
    {
        public new event ScrollEventHandler Scroll;
        public event EventHandler ValueChanged;

        private Rectangle _trackRectangle = new Rectangle();
        private Rectangle _thumbRectangle = new Rectangle();
        private long _value, _min, _max;
        private TrackBarThumbState _thumbState;

        public Slider()
        {
            InitializeComponent();
            SetStyle(ControlStyles.UserMouse | ControlStyles.SupportsTransparentBackColor | ControlStyles.UserPaint, true);
            this.UpdateStyles();
            _max = 10;
        }

        private Rectangle GetThumbRectangle()
        {
            int halfheight = Height / 2;
            return new Rectangle(new Point((int)(2 + (Width - 14) * ((double)_value / (double)_max)), halfheight - 10), new Size(10, 20));
        }

        private Rectangle GetTrackRectangle()
        {
            int halfheight = Height / 2;
            return new Rectangle(5, halfheight - 2, Width - 10, 4);
        }

        // Draw the track bar.
        protected override void OnPaint(PaintEventArgs e)
        {
            if (!TrackBarRenderer.IsSupported)
            {
                return;
            }

            TrackBarRenderer.DrawHorizontalTrack(e.Graphics,
                trackRectangle);
            if (Enabled)
                TrackBarRenderer.DrawHorizontalThumb(e.Graphics,
                    thumbRectangle, thumbState);
            else
                TrackBarRenderer.DrawHorizontalThumb(e.Graphics,
                    thumbRectangle, TrackBarThumbState.Disabled);
        }

        // Determine whether the user has clicked the track bar thumb.
        protected override void OnMouseDown(MouseEventArgs e)
        {
            SetValue(e);
            OnScroll(new ScrollEventArgs(ScrollEventType.ThumbTrack, 0));
            if (this.thumbRectangle.Contains(e.Location))
            {
                thumbState = TrackBarThumbState.Pressed;
            }
        }

        // Redraw the track bar thumb if the user has moved it.
        protected override void OnMouseUp(MouseEventArgs e)
        {
            if (!TrackBarRenderer.IsSupported)
                return;

            SetValue(e);
            OnScroll(new ScrollEventArgs(ScrollEventType.ThumbTrack, 0));

        }

        // Track cursor movements.
        protected override void OnMouseMove(MouseEventArgs e)
        {
            if (!TrackBarRenderer.IsSupported)
                return;



            if ((e.Button & MouseButtons.Left) != 0)
            {
                SetValue(e);
                OnScroll(new ScrollEventArgs(ScrollEventType.ThumbTrack, 0));
            }
            // The cursor is passing over the track.
            else
            {
                thumbState = thumbRectangle.Contains(e.Location) ?
                    TrackBarThumbState.Hot : TrackBarThumbState.Normal;
            }

        }

        private void SetValue(MouseEventArgs e)
        {
            double tmpval = ((double)(e.X - 5) / ((double)Width - 10) * ((double)Max - Min) + Min);
            if (tmpval < Min)
                tmpval = Min;
            if (tmpval > Max)
                tmpval = Max;
            if (Min >= Max)
                tmpval = Min;
            Value = (long)Math.Ceiling(tmpval - 0.5);
        }

        protected override void OnResize(EventArgs e)
        {
            trackRectangle = GetTrackRectangle();
            thumbRectangle = GetThumbRectangle();
            base.OnResize(e);
        }

        private Rectangle trackRectangle
        {
            get
            {
                return _trackRectangle;
            }
            set
            {
                if (_trackRectangle == value)
                    return;
                _trackRectangle = value;
                Invalidate();
            }
        }

        private Rectangle thumbRectangle
        {
            get
            {
                return _thumbRectangle;
            }
            set
            {
                if (_thumbRectangle == value)
                    return;
                _thumbRectangle = value;
                Invalidate();
            }
        }

        private TrackBarThumbState thumbState
        {
            get
            {
                return _thumbState;
            }
            set
            {
                if (_thumbState == value)
                    return;
                _thumbState = value;
                Invalidate();
            }
        }

        public long Value
        {
            get
            {
                return _value;
            }
            set
            {
                if (_value == value)
                    return;
                if(_value < _min || value > _max)
                    throw new ArgumentException("Value must be >= Min and <= Max");
                _value = value;
                OnValueChanged(new EventArgs());
                thumbRectangle = GetThumbRectangle();
            }
        }

        public long Min
        {
            get
            {
                return _min;
            }
            set
            {
                if (_min == value)
                    return;
                _min = value;
                thumbRectangle = GetThumbRectangle();
            }
        }

        public long Max
        {
            get
            {
                return _max;
            }
            set
            {
                if (_max == value)
                    return;
                _max = value;
                thumbRectangle = GetThumbRectangle();
            }
        }

        protected override void OnScroll(ScrollEventArgs se)
        {
            if (Scroll != null)
                Scroll(this, se);
        }

        protected virtual void OnValueChanged(EventArgs e)
        {
            if (ValueChanged != null)
                ValueChanged(this, e);
        }
    }

}
